@@ -14,10 +14,10 @@
 import math
 from dataclasses import dataclass
 from datetime import timedelta
-from typing import Any, Optional, Union
+from typing import Any, Dict, Optional, Union
 
+import pytorch_lightning as pl
 from pytorch_lightning.callbacks.progress.base import ProgressBarBase
-from pytorch_lightning.utilities.exceptions import MisconfigurationException
 from pytorch_lightning.utilities.imports import _RICH_AVAILABLE
 
 Task, Style = None, None
@@ -145,10 +145,12 @@ if _RICH_AVAILABLE:
             self._metrics = metrics
 
         def render(self, task) -> Text:
-            from pytorch_lightning.trainer.states import TrainerFn
-
-            if self._trainer.state.fn != TrainerFn.FITTING or self._trainer.sanity_checking:
-                return Text("")
+            if (
+                self._trainer.state.fn != "fit"
+                or self._trainer.sanity_checking
+                or self._trainer.progress_bar_callback.main_progress_bar_id != task.id
+            ):
+                return Text()
             if self._trainer.training and task.id not in self._tasks:
                 self._tasks[task.id] = "None"
                 if self._renderable_cache:
@@ -156,11 +158,11 @@ if _RICH_AVAILABLE:
                 self._current_task_id = task.id
             if self._trainer.training and task.id != self._current_task_id:
                 return self._tasks[task.id]
-            _text = ""
 
+            text = ""
             for k, v in self._metrics.items():
-                _text += f"{k}: {round(v, 3) if isinstance(v, float) else v} "
-            return Text(_text, justify="left", style=self._style)
+                text += f"{k}: {round(v, 3) if isinstance(v, float) else v} "
+            return Text(text, justify="left", style=self._style)
 
 
 @dataclass
@@ -207,29 +209,38 @@ class RichProgressBar(ProgressBarBase):
         trainer = Trainer(callbacks=RichProgressBar())
 
     Args:
-        refresh_rate_per_second: the number of updates per second. If refresh_rate is 0, progress bar is disabled.
+        refresh_rate: Determines at which rate (in number of batches) the progress bars get updated.
+            Set it to ``0`` to disable the display.
         leave: Leaves the finished progress bar in the terminal at the end of the epoch. Default: False
         theme: Contains styles used to stylize the progress bar.
+        console_kwargs: Args for constructing a `Console`
 
     Raises:
         ModuleNotFoundError:
             If required `rich` package is not installed on the device.
+
+    Note:
+        PyCharm users will need to enable “emulate terminal” in output console option in
+        run/debug configuration to see styled output.
+        Reference: https://rich.readthedocs.io/en/latest/introduction.html#requirements
     """
 
     def __init__(
         self,
-        refresh_rate_per_second: int = 10,
+        refresh_rate: int = 1,
         leave: bool = False,
         theme: RichProgressBarTheme = RichProgressBarTheme(),
+        console_kwargs: Optional[Dict[str, Any]] = None,
     ) -> None:
         if not _RICH_AVAILABLE:
-            raise MisconfigurationException(
+            raise ModuleNotFoundError(
                 "`RichProgressBar` requires `rich` >= 10.2.2. Install it by running `pip install -U rich`."
             )
 
         super().__init__()
-        self._refresh_rate_per_second: int = refresh_rate_per_second
+        self._refresh_rate: int = refresh_rate
         self._leave: bool = leave
+        self._console_kwargs = console_kwargs or {}
         self._enabled: bool = True
         self.progress: Optional[Progress] = None
         self.val_sanity_progress_bar_id: Optional[int] = None
@@ -237,56 +248,43 @@ class RichProgressBar(ProgressBarBase):
         self._metric_component = None
         self._progress_stopped: bool = False
         self.theme = theme
+        self._update_for_light_colab_theme()
 
     @property
-    def refresh_rate_per_second(self) -> float:
-        """Refresh rate for Rich Progress.
-
-        Returns: Refresh rate for Progress Bar.
-            Return 1 if not enabled, as a positive integer is required (ignored by Rich Progress).
-        """
-        return self._refresh_rate_per_second if self._refresh_rate_per_second > 0 else 1
+    def refresh_rate(self) -> float:
+        return self._refresh_rate
 
     @property
     def is_enabled(self) -> bool:
-        return self._enabled and self._refresh_rate_per_second > 0
+        return self._enabled and self.refresh_rate > 0
 
     @property
     def is_disabled(self) -> bool:
         return not self.is_enabled
 
+    def _update_for_light_colab_theme(self) -> None:
+        if _detect_light_colab_theme():
+            attributes = ["description", "batch_progress", "metrics"]
+            for attr in attributes:
+                if getattr(self.theme, attr) == "white":
+                    setattr(self.theme, attr, "black")
+
     def disable(self) -> None:
         self._enabled = False
 
     def enable(self) -> None:
         self._enabled = True
 
-    @property
-    def sanity_check_description(self) -> str:
-        return "Validation Sanity Check"
-
-    @property
-    def validation_description(self) -> str:
-        return "Validation"
-
-    @property
-    def test_description(self) -> str:
-        return "Testing"
-
-    @property
-    def predict_description(self) -> str:
-        return "Predicting"
-
     def _init_progress(self, trainer):
         if self.is_enabled and (self.progress is None or self._progress_stopped):
             self._reset_progress_bar_ids()
-            self._console: Console = Console()
+            self._console = Console(**self._console_kwargs)
             self._console.clear_live()
             self._metric_component = MetricsTextColumn(trainer, self.theme.metrics)
             self.progress = CustomProgress(
                 *self.configure_columns(trainer),
                 self._metric_component,
-                refresh_per_second=self.refresh_rate_per_second,
+                auto_refresh=False,
                 disable=self.is_disabled,
                 console=self._console,
             )
@@ -294,46 +292,31 @@ class RichProgressBar(ProgressBarBase):
             # progress has started
             self._progress_stopped = False
 
+    def refresh(self) -> None:
+        if self.progress:
+            self.progress.refresh()
+
     def on_train_start(self, trainer, pl_module):
-        super().on_train_start(trainer, pl_module)
         self._init_progress(trainer)
 
     def on_predict_start(self, trainer, pl_module):
-        super().on_predict_start(trainer, pl_module)
         self._init_progress(trainer)
 
     def on_test_start(self, trainer, pl_module):
-        super().on_test_start(trainer, pl_module)
         self._init_progress(trainer)
 
     def on_validation_start(self, trainer, pl_module):
-        super().on_validation_start(trainer, pl_module)
         self._init_progress(trainer)
 
-    def __getstate__(self):
-        # can't pickle the rich progress objects
-        state = self.__dict__.copy()
-        state["progress"] = None
-        state["_console"] = None
-        return state
-
-    def __setstate__(self, state):
-        self.__dict__ = state
-        state["_console"] = Console()
-
     def on_sanity_check_start(self, trainer, pl_module):
-        super().on_sanity_check_start(trainer, pl_module)
         self._init_progress(trainer)
-        self.val_sanity_progress_bar_id = self._add_task(trainer.num_sanity_val_steps, self.sanity_check_description)
 
     def on_sanity_check_end(self, trainer, pl_module):
-        super().on_sanity_check_end(trainer, pl_module)
         if self.progress is not None:
             self.progress.update(self.val_sanity_progress_bar_id, advance=0, visible=False)
-            self.progress.refresh()
+        self.refresh()
 
     def on_train_epoch_start(self, trainer, pl_module):
-        super().on_train_epoch_start(trainer, pl_module)
         total_train_batches = self.total_train_batches
         total_val_batches = self.total_val_batches
         if total_train_batches != float("inf"):
@@ -353,16 +336,31 @@ class RichProgressBar(ProgressBarBase):
             self.progress.reset(
                 self.main_progress_bar_id, total=total_batches, description=train_description, visible=True
             )
+        self.refresh()
 
-    def on_validation_epoch_start(self, trainer, pl_module):
-        super().on_validation_epoch_start(trainer, pl_module)
-        if self.total_val_batches > 0:
-            total_val_batches = self.total_val_batches
-            if self.total_train_batches != float("inf") and hasattr(trainer, "val_check_batch"):
-                # val can be checked multiple times per epoch
-                val_checks_per_epoch = self.total_train_batches // trainer.val_check_batch
-                total_val_batches = self.total_val_batches * val_checks_per_epoch
-            self.val_progress_bar_id = self._add_task(total_val_batches, self.validation_description, visible=False)
+    def on_validation_batch_start(
+        self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", batch: Any, batch_idx: int, dataloader_idx: int
+    ) -> None:
+        if not self.has_dataloader_changed(dataloader_idx):
+            return
+
+        if trainer.sanity_checking:
+            if self.val_sanity_progress_bar_id is not None:
+                self.progress.update(self.val_sanity_progress_bar_id, advance=0, visible=False)
+
+            self.val_sanity_progress_bar_id = self._add_task(
+                self.total_val_batches_current_dataloader, self.sanity_check_description, visible=False
+            )
+        else:
+            if self.val_progress_bar_id is not None:
+                self.progress.update(self.val_progress_bar_id, advance=0, visible=False)
+
+            # TODO: remove old tasks when new onces are created
+            self.val_progress_bar_id = self._add_task(
+                self.total_val_batches_current_dataloader, self.validation_description, visible=False
+            )
+
+        self.refresh()
 
     def _add_task(self, total_batches: int, description: str, visible: bool = True) -> Optional[int]:
         if self.progress is not None:
@@ -370,45 +368,84 @@ class RichProgressBar(ProgressBarBase):
                 f"[{self.theme.description}]{description}", total=total_batches, visible=visible
             )
 
-    def _update(self, progress_bar_id: int, visible: bool = True) -> None:
-        if self.progress is not None:
-            self.progress.update(progress_bar_id, advance=1.0, visible=visible)
+    def _update(self, progress_bar_id: int, current: int, total: Union[int, float], visible: bool = True) -> None:
+        if self.progress is not None and self._should_update(current, total):
+            leftover = current % self.refresh_rate
+            advance = leftover if (current == total and leftover != 0) else self.refresh_rate
+            self.progress.update(progress_bar_id, advance=advance, visible=visible)
+            self.refresh()
+
+    def _should_update(self, current: int, total: Union[int, float]) -> bool:
+        return self.is_enabled and (current % self.refresh_rate == 0 or current == total)
 
     def on_validation_epoch_end(self, trainer, pl_module):
-        super().on_validation_epoch_end(trainer, pl_module)
-        if self.val_progress_bar_id is not None:
-            self._update(self.val_progress_bar_id, visible=False)
+        if self.val_progress_bar_id is not None and trainer.state.fn == "fit":
+            self.progress.update(self.val_progress_bar_id, advance=0, visible=False)
+            self.refresh()
 
-    def on_test_epoch_start(self, trainer, pl_module):
-        super().on_train_epoch_start(trainer, pl_module)
-        self.test_progress_bar_id = self._add_task(self.total_test_batches, self.test_description)
+    def on_validation_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
+        if trainer.state.fn == "fit":
+            self._update_metrics(trainer, pl_module)
+        self.reset_dataloader_idx_tracker()
 
-    def on_predict_epoch_start(self, trainer, pl_module):
-        super().on_predict_epoch_start(trainer, pl_module)
-        self.predict_progress_bar_id = self._add_task(self.total_predict_batches, self.predict_description)
+    def on_test_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
+        self.reset_dataloader_idx_tracker()
+
+    def on_predict_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
+        self.reset_dataloader_idx_tracker()
+
+    def on_test_batch_start(
+        self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", batch: Any, batch_idx: int, dataloader_idx: int
+    ) -> None:
+        if not self.has_dataloader_changed(dataloader_idx):
+            return
+
+        if self.test_progress_bar_id is not None:
+            self.progress.update(self.test_progress_bar_id, advance=0, visible=False)
+        self.test_progress_bar_id = self._add_task(self.total_test_batches_current_dataloader, self.test_description)
+        self.refresh()
+
+    def on_predict_batch_start(
+        self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", batch: Any, batch_idx: int, dataloader_idx: int
+    ) -> None:
+        if not self.has_dataloader_changed(dataloader_idx):
+            return
+
+        if self.predict_progress_bar_id is not None:
+            self.progress.update(self.predict_progress_bar_id, advance=0, visible=False)
+        self.predict_progress_bar_id = self._add_task(
+            self.total_predict_batches_current_dataloader, self.predict_description
+        )
+        self.refresh()
 
     def on_train_batch_end(self, trainer, pl_module, outputs, batch, batch_idx):
-        super().on_train_batch_end(trainer, pl_module, outputs, batch, batch_idx)
-        self._update(self.main_progress_bar_id)
+        self._update(self.main_progress_bar_id, self.train_batch_idx, self.total_train_batches)
+        self._update_metrics(trainer, pl_module)
+        self.refresh()
+
+    def on_train_epoch_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
         self._update_metrics(trainer, pl_module)
 
     def on_validation_batch_end(self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx):
-        super().on_validation_batch_end(trainer, pl_module, outputs, batch, batch_idx, dataloader_idx)
         if trainer.sanity_checking:
-            self._update(self.val_sanity_progress_bar_id)
+            self._update(self.val_sanity_progress_bar_id, self.val_batch_idx, self.total_val_batches_current_dataloader)
         elif self.val_progress_bar_id is not None:
             # check to see if we should update the main training progress bar
             if self.main_progress_bar_id is not None:
-                self._update(self.main_progress_bar_id)
-            self._update(self.val_progress_bar_id)
+                # TODO: Use total val_processed here just like TQDM in a follow-up
+                self._update(self.main_progress_bar_id, self.val_batch_idx, self.total_val_batches_current_dataloader)
+            self._update(self.val_progress_bar_id, self.val_batch_idx, self.total_val_batches_current_dataloader)
+        self.refresh()
 
     def on_test_batch_end(self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx):
-        super().on_test_batch_end(trainer, pl_module, outputs, batch, batch_idx, dataloader_idx)
-        self._update(self.test_progress_bar_id)
+        self._update(self.test_progress_bar_id, self.test_batch_idx, self.total_test_batches_current_dataloader)
+        self.refresh()
 
     def on_predict_batch_end(self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx):
-        super().on_predict_batch_end(trainer, pl_module, outputs, batch, batch_idx, dataloader_idx)
-        self._update(self.predict_progress_bar_id)
+        self._update(
+            self.predict_progress_bar_id, self.predict_batch_idx, self.total_predict_batches_current_dataloader
+        )
+        self.refresh()
 
     def _get_train_description(self, current_epoch: int) -> str:
         train_description = f"Epoch {current_epoch}"
@@ -472,3 +509,20 @@ class RichProgressBar(ProgressBarBase):
             CustomTimeColumn(style=self.theme.time),
             ProcessingSpeedColumn(style=self.theme.processing_speed),
         ]
+
+
+def _detect_light_colab_theme() -> bool:
+    """Detect if it's light theme in Colab."""
+    try:
+        get_ipython  # type: ignore
+    except NameError:
+        return False
+    ipython = get_ipython()  # noqa: F821
+    if "google.colab" in str(ipython.__class__):
+        try:
+            from google.colab import output
+
+            return output.eval_js('document.documentElement.matches("[theme=light]")')
+        except ModuleNotFoundError:
+            return False
+    return False
