@@ -12,14 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import importlib
-import io
 import math
 import os
 import sys
-from typing import Optional, Union
+from typing import Any, Dict, Optional, Union
 
 # check if ipywidgets is installed before importing tqdm.auto
 # to ensure it won't fail and a progress bar is displayed
+
 if importlib.util.find_spec("ipywidgets") is not None:
     from tqdm.auto import tqdm as _tqdm
 else:
@@ -27,23 +27,25 @@ else:
 
 import pytorch_lightning as pl
 from pytorch_lightning.callbacks.progress.base import ProgressBarBase
+from pytorch_lightning.utilities.rank_zero import rank_zero_debug
 
 _PAD_SIZE = 5
 
 
 class Tqdm(_tqdm):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Custom tqdm progressbar where we append 0 to floating points/strings to prevent the progress bar from
         flickering."""
         # this just to make the make docs happy, otherwise it pulls docs which has some issues...
         super().__init__(*args, **kwargs)
 
     @staticmethod
-    def format_num(n) -> str:
+    def format_num(n: Union[int, float, str]) -> str:
         """Add additional padding to the formatted numbers."""
         should_be_padded = isinstance(n, (float, str))
         if not isinstance(n, str):
             n = _tqdm.format_num(n)
+            assert isinstance(n, str)
         if should_be_padded and "e" not in n:
             if "." not in n and len(n) < _PAD_SIZE:
                 try:
@@ -101,22 +103,57 @@ class TQDMProgressBar(ProgressBarBase):
 
     def __init__(self, refresh_rate: int = 1, process_position: int = 0):
         super().__init__()
-        self._refresh_rate = refresh_rate
+        self._refresh_rate = self._resolve_refresh_rate(refresh_rate)
         self._process_position = process_position
         self._enabled = True
-        self.main_progress_bar = None
-        self.val_progress_bar = None
-        self.test_progress_bar = None
-        self.predict_progress_bar = None
+        self._main_progress_bar: Optional[_tqdm] = None
+        self._val_progress_bar: Optional[_tqdm] = None
+        self._test_progress_bar: Optional[_tqdm] = None
+        self._predict_progress_bar: Optional[_tqdm] = None
 
-    def __getstate__(self):
+    def __getstate__(self) -> Dict:
         # can't pickle the tqdm objects
-        state = self.__dict__.copy()
-        state["main_progress_bar"] = None
-        state["val_progress_bar"] = None
-        state["test_progress_bar"] = None
-        state["predict_progress_bar"] = None
-        return state
+        return {k: v if not isinstance(v, _tqdm) else None for k, v in vars(self).items()}
+
+    @property
+    def main_progress_bar(self) -> _tqdm:
+        if self._main_progress_bar is None:
+            raise TypeError(f"The `{self.__class__.__name__}._main_progress_bar` reference has not been set yet.")
+        return self._main_progress_bar
+
+    @main_progress_bar.setter
+    def main_progress_bar(self, bar: _tqdm) -> None:
+        self._main_progress_bar = bar
+
+    @property
+    def val_progress_bar(self) -> _tqdm:
+        if self._val_progress_bar is None:
+            raise TypeError(f"The `{self.__class__.__name__}._val_progress_bar` reference has not been set yet.")
+        return self._val_progress_bar
+
+    @val_progress_bar.setter
+    def val_progress_bar(self, bar: _tqdm) -> None:
+        self._val_progress_bar = bar
+
+    @property
+    def test_progress_bar(self) -> _tqdm:
+        if self._test_progress_bar is None:
+            raise TypeError(f"The `{self.__class__.__name__}._test_progress_bar` reference has not been set yet.")
+        return self._test_progress_bar
+
+    @test_progress_bar.setter
+    def test_progress_bar(self, bar: _tqdm) -> None:
+        self._test_progress_bar = bar
+
+    @property
+    def predict_progress_bar(self) -> _tqdm:
+        if self._predict_progress_bar is None:
+            raise TypeError(f"The `{self.__class__.__name__}._predict_progress_bar` reference has not been set yet.")
+        return self._predict_progress_bar
+
+    @predict_progress_bar.setter
+    def predict_progress_bar(self, bar: _tqdm) -> None:
+        self._predict_progress_bar = bar
 
     @property
     def refresh_rate(self) -> int:
@@ -134,6 +171,11 @@ class TQDMProgressBar(ProgressBarBase):
     def is_disabled(self) -> bool:
         return not self.is_enabled
 
+    @property
+    def _val_processed(self) -> int:
+        # use total in case validation runs more than once per training epoch
+        return self.trainer.fit_loop.epoch_loop.val_loop.epoch_loop.batch_progress.total.processed
+
     def disable(self) -> None:
         self._enabled = False
 
@@ -143,7 +185,7 @@ class TQDMProgressBar(ProgressBarBase):
     def init_sanity_tqdm(self) -> Tqdm:
         """Override this to customize the tqdm bar for the validation sanity run."""
         bar = Tqdm(
-            desc="Validation sanity check",
+            desc=self.sanity_check_description,
             position=(2 * self.process_position),
             disable=self.is_disabled,
             leave=False,
@@ -155,7 +197,7 @@ class TQDMProgressBar(ProgressBarBase):
     def init_train_tqdm(self) -> Tqdm:
         """Override this to customize the tqdm bar for training."""
         bar = Tqdm(
-            desc="Training",
+            desc=self.train_description,
             initial=self.train_batch_idx,
             position=(2 * self.process_position),
             disable=self.is_disabled,
@@ -169,7 +211,7 @@ class TQDMProgressBar(ProgressBarBase):
     def init_predict_tqdm(self) -> Tqdm:
         """Override this to customize the tqdm bar for predicting."""
         bar = Tqdm(
-            desc="Predicting",
+            desc=self.predict_description,
             initial=self.train_batch_idx,
             position=(2 * self.process_position),
             disable=self.is_disabled,
@@ -183,12 +225,12 @@ class TQDMProgressBar(ProgressBarBase):
     def init_validation_tqdm(self) -> Tqdm:
         """Override this to customize the tqdm bar for validation."""
         # The main progress bar doesn't exist in `trainer.validate()`
-        has_main_bar = self.main_progress_bar is not None
+        has_main_bar = self.trainer.state.fn != "validate"
         bar = Tqdm(
-            desc="Validating",
+            desc=self.validation_description,
             position=(2 * self.process_position + has_main_bar),
             disable=self.is_disabled,
-            leave=False,
+            leave=not has_main_bar,
             dynamic_ncols=True,
             file=sys.stdout,
         )
@@ -206,20 +248,18 @@ class TQDMProgressBar(ProgressBarBase):
         )
         return bar
 
-    def on_sanity_check_start(self, trainer, pl_module):
+    def on_sanity_check_start(self, *_: Any) -> None:
         self.val_progress_bar = self.init_sanity_tqdm()
         self.main_progress_bar = Tqdm(disable=True)  # dummy progress bar
 
-    def on_sanity_check_end(self, trainer, pl_module):
+    def on_sanity_check_end(self, *_: Any) -> None:
         self.main_progress_bar.close()
         self.val_progress_bar.close()
 
-    def on_train_start(self, trainer, pl_module):
-        super().on_train_start(trainer, pl_module)
+    def on_train_start(self, *_: Any) -> None:
         self.main_progress_bar = self.init_train_tqdm()
 
-    def on_train_epoch_start(self, trainer, pl_module):
-        super().on_train_epoch_start(trainer, pl_module)
+    def on_train_epoch_start(self, trainer: "pl.Trainer", *_: Any) -> None:
         total_train_batches = self.total_train_batches
         total_val_batches = self.total_val_batches
         if total_train_batches != float("inf") and total_val_batches != float("inf"):
@@ -227,111 +267,113 @@ class TQDMProgressBar(ProgressBarBase):
             val_checks_per_epoch = total_train_batches // trainer.val_check_batch
             total_val_batches = total_val_batches * val_checks_per_epoch
         total_batches = total_train_batches + total_val_batches
-        reset(self.main_progress_bar, total=total_batches, current=self.train_batch_idx)
+        self.main_progress_bar.total = convert_inf(total_batches)
         self.main_progress_bar.set_description(f"Epoch {trainer.current_epoch}")
 
-    def on_train_batch_end(self, trainer, pl_module, outputs, batch, batch_idx):
-        super().on_train_batch_end(trainer, pl_module, outputs, batch, batch_idx)
-        if self._should_update(self.train_batch_idx):
-            self._update_bar(self.main_progress_bar)
+    def on_train_batch_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", *_: Any) -> None:
+        if self._should_update(self.train_batch_idx, self.total_train_batches):
+            _update_n(self.main_progress_bar, self.train_batch_idx + self._val_processed)
             self.main_progress_bar.set_postfix(self.get_metrics(trainer, pl_module))
 
     def on_train_epoch_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
-        if self.is_enabled:
-            self._update_bar(self.main_progress_bar)
+        if not self.main_progress_bar.disable:
             self.main_progress_bar.set_postfix(self.get_metrics(trainer, pl_module))
 
-    def on_train_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
+    def on_train_end(self, *_: Any) -> None:
         self.main_progress_bar.close()
 
-    def on_validation_start(self, trainer, pl_module):
-        super().on_validation_start(trainer, pl_module)
-        if trainer.sanity_checking:
-            reset(self.val_progress_bar, total=sum(trainer.num_sanity_val_batches), current=self.val_batch_idx)
-        else:
-            if trainer.state.fn == pl.trainer.states.TrainerFn.FITTING:
-                self._update_bar(self.main_progress_bar)  # fill up remaining
+    def on_validation_start(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
+        if not trainer.sanity_checking:
             self.val_progress_bar = self.init_validation_tqdm()
-            reset(self.val_progress_bar, total=self.total_val_batches, current=self.val_batch_idx)
 
-    def on_validation_batch_end(self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx):
-        super().on_validation_batch_end(trainer, pl_module, outputs, batch, batch_idx, dataloader_idx)
-        if self._should_update(self.val_batch_idx):
-            self._update_bar(self.val_progress_bar)
-            if trainer.state.fn == pl.trainer.states.TrainerFn.FITTING:
-                self._update_bar(self.main_progress_bar)
+    def on_validation_batch_start(
+        self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", batch: Any, batch_idx: int, dataloader_idx: int
+    ) -> None:
+        if not self.has_dataloader_changed(dataloader_idx):
+            return
 
-    def on_validation_epoch_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
-        if self.is_enabled:
-            self._update_bar(self.val_progress_bar)
+        self.val_progress_bar.total = convert_inf(self.total_val_batches_current_dataloader)
+        desc = self.sanity_check_description if trainer.sanity_checking else self.validation_description
+        self.val_progress_bar.set_description(f"{desc} DataLoader {dataloader_idx}")
 
-    def on_validation_end(self, trainer, pl_module):
-        if self.main_progress_bar is not None and trainer.state.fn == pl.trainer.states.TrainerFn.FITTING:
+    def on_validation_batch_end(self, trainer: "pl.Trainer", *_: Any) -> None:
+        if self._should_update(self.val_batch_idx, self.total_val_batches_current_dataloader):
+            _update_n(self.val_progress_bar, self.val_batch_idx)
+            if trainer.state.fn == "fit":
+                _update_n(self.main_progress_bar, self.train_batch_idx + self._val_processed)
+
+    def on_validation_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
+        if self._main_progress_bar is not None and trainer.state.fn == "fit":
             self.main_progress_bar.set_postfix(self.get_metrics(trainer, pl_module))
         self.val_progress_bar.close()
+        self.reset_dataloader_idx_tracker()
 
-    def on_test_start(self, trainer, pl_module):
-        super().on_test_start(trainer, pl_module)
+    def on_test_start(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
         self.test_progress_bar = self.init_test_tqdm()
-        self.test_progress_bar.total = convert_inf(self.total_test_batches)
 
-    def on_test_batch_end(self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx):
-        super().on_test_batch_end(trainer, pl_module, outputs, batch, batch_idx, dataloader_idx)
-        if self._should_update(self.test_batch_idx):
-            self._update_bar(self.test_progress_bar)
+    def on_test_batch_start(
+        self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", batch: Any, batch_idx: int, dataloader_idx: int
+    ) -> None:
+        if not self.has_dataloader_changed(dataloader_idx):
+            return
 
-    def on_test_epoch_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
-        if self.is_enabled:
-            self._update_bar(self.test_progress_bar)
+        self.test_progress_bar.total = convert_inf(self.total_test_batches_current_dataloader)
+        self.test_progress_bar.set_description(f"{self.test_description} DataLoader {dataloader_idx}")
 
-    def on_test_end(self, trainer, pl_module):
+    def on_test_batch_end(self, *_: Any) -> None:
+        if self._should_update(self.test_batch_idx, self.total_test_batches_current_dataloader):
+            _update_n(self.test_progress_bar, self.test_batch_idx)
+
+    def on_test_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
         self.test_progress_bar.close()
+        self.reset_dataloader_idx_tracker()
 
-    def on_predict_epoch_start(self, trainer, pl_module):
-        super().on_predict_epoch_start(trainer, pl_module)
+    def on_predict_start(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
         self.predict_progress_bar = self.init_predict_tqdm()
-        self.predict_progress_bar.total = convert_inf(self.total_predict_batches)
 
-    def on_predict_batch_end(self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx):
-        super().on_predict_batch_end(trainer, pl_module, outputs, batch, batch_idx, dataloader_idx)
-        if self._should_update(self.predict_batch_idx):
-            self._update_bar(self.predict_progress_bar)
+    def on_predict_batch_start(
+        self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", batch: Any, batch_idx: int, dataloader_idx: int
+    ) -> None:
+        if not self.has_dataloader_changed(dataloader_idx):
+            return
 
-    def on_predict_end(self, trainer, pl_module):
+        self.predict_progress_bar.total = convert_inf(self.total_predict_batches_current_dataloader)
+        self.predict_progress_bar.set_description(f"{self.predict_description} DataLoader {dataloader_idx}")
+
+    def on_predict_batch_end(self, *_: Any) -> None:
+        if self._should_update(self.predict_batch_idx, self.total_predict_batches_current_dataloader):
+            _update_n(self.predict_progress_bar, self.predict_batch_idx)
+
+    def on_predict_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
         self.predict_progress_bar.close()
+        self.reset_dataloader_idx_tracker()
 
-    def print(
-        self, *args, sep: str = " ", end: str = os.linesep, file: Optional[io.TextIOBase] = None, nolock: bool = False
-    ):
+    def print(self, *args: Any, sep: str = " ", **kwargs: Any) -> None:
         active_progress_bar = None
 
-        if self.main_progress_bar is not None and not self.main_progress_bar.disable:
+        if self._main_progress_bar is not None and not self.main_progress_bar.disable:
             active_progress_bar = self.main_progress_bar
-        elif self.val_progress_bar is not None and not self.val_progress_bar.disable:
+        elif self._val_progress_bar is not None and not self.val_progress_bar.disable:
             active_progress_bar = self.val_progress_bar
-        elif self.test_progress_bar is not None and not self.test_progress_bar.disable:
+        elif self._test_progress_bar is not None and not self.test_progress_bar.disable:
             active_progress_bar = self.test_progress_bar
-        elif self.predict_progress_bar is not None and not self.predict_progress_bar.disable:
+        elif self._predict_progress_bar is not None and not self.predict_progress_bar.disable:
             active_progress_bar = self.predict_progress_bar
 
         if active_progress_bar is not None:
             s = sep.join(map(str, args))
-            active_progress_bar.write(s, end=end, file=file, nolock=nolock)
+            active_progress_bar.write(s, **kwargs)
 
-    def _should_update(self, idx: int) -> bool:
-        return self.is_enabled and (idx % self.refresh_rate == 0)
+    def _should_update(self, current: int, total: Union[int, float]) -> bool:
+        return self.refresh_rate > 0 and (current % self.refresh_rate == 0 or current == total)
 
-    def _update_bar(self, bar: Optional[Tqdm]) -> None:
-        """Updates the bar by the refresh rate without overshooting."""
-        if bar is None:
-            return
-        if bar.total is not None:
-            delta = min(self.refresh_rate, bar.total - bar.n)
-        else:
-            # infinite / unknown size
-            delta = self.refresh_rate
-        if delta > 0:
-            bar.update(delta)
+    @staticmethod
+    def _resolve_refresh_rate(refresh_rate: int) -> int:
+        if os.getenv("COLAB_GPU") and refresh_rate == 1:
+            # smaller refresh rate on colab causes crashes, choose a higher value
+            rank_zero_debug("Using a higher refresh rate on Colab. Setting it to `20`")
+            refresh_rate = 20
+        return refresh_rate
 
 
 def convert_inf(x: Optional[Union[int, float]]) -> Optional[Union[int, float]]:
@@ -344,8 +386,7 @@ def convert_inf(x: Optional[Union[int, float]]) -> Optional[Union[int, float]]:
     return x
 
 
-def reset(bar: Tqdm, total: Optional[int] = None, current: int = 0) -> None:
-    """Resets the tqdm bar to the desired position and sets a new total, unless it is disabled."""
+def _update_n(bar: _tqdm, value: int) -> None:
     if not bar.disable:
-        bar.reset(total=convert_inf(total))
-        bar.n = current
+        bar.n = value
+        bar.refresh()
