@@ -13,7 +13,8 @@
 # limitations under the License.
 from typing import Any
 
-from pytorch_lightning.overrides.torch_distributed import broadcast_object_list
+import torch.distributed
+
 from pytorch_lightning.utilities import rank_zero_deprecation
 from pytorch_lightning.utilities.distributed import group as _group
 
@@ -22,13 +23,14 @@ class LightningDistributed:
     """
     .. deprecated:: v1.5
         This class is deprecated in v1.5 and will be removed in v1.7.
-        The broadcast logic will be moved to the :class:`DDPPlugin` and :class`DDPSpawnPlugin` classes.
+        The broadcast logic will be moved to the :class:`DDPStrategy` and :class`DDPSpawnStrategy` classes.
+
     """
 
     def __init__(self, rank=None, device=None):
         rank_zero_deprecation(
             "LightningDistributed is deprecated in v1.5 and will be removed in v1.7."
-            "Broadcast logic is implemented directly in the :class:`TrainingTypePlugin` implementations."
+            "Broadcast logic is implemented directly in the :class:`Strategy` implementations."
         )
         self.rank = rank
         self.device = device
@@ -40,6 +42,6 @@ class LightningDistributed:
         if self.rank != 0:
             obj = [None] * len(obj)
 
-        broadcast_object_list(obj, 0, group=group or _group.WORLD)
+        torch.distributed.broadcast_object_list(obj, 0, group=group or _group.WORLD)
 
         return obj[0]
